/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.SetupUsers");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.validate.regexp");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.client.internal.userRegistry");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.LdapRealm");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");
dojo.require("com.ibm.team.repository.web.client.session.Session");

(function() {
	

var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ConfigUtility = com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var RepositoryClient = com.ibm.team.repository.web.client.internal.RepositoryClient;
var LicenseAdminClient = com.ibm.team.repository.web.client.internal.licensing;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var userRegistry = com.ibm.team.repository.web.client.internal.userRegistry;
var arrayIndexWithPropertyOfObject = com.ibm.team.repository.web.ui.internal.admin.util.arrayIndexWithPropertyOfObject;
var isDefaultAdmin = com.ibm.team.repository.web.client.session.isDefaultAdmin;

var PACKAGE_PREFIX = "com.ibm.team.repository.web.ui.internal.wizard.";
var REGISTRY_SERVICE_NAME = "com.ibm.team.repository.service.internal.userregistry.ExternalUserRegistryService";
var REGISTRY_REST_SERVICE_INTERFACE_NAME = "com.ibm.team.repository.service.internal.IExternalUserRegistryRestService"
var REGISTRY_SERVICE_INTERFACE_NAME = "com.ibm.team.repository.common.service.IExternalUserRegistryService";
var REGISTRY_PROPERTY_NAME = "user.registry.type";
var REPO_ITEM_SERVICE_INTERFACE_NAME = "com.ibm.team.repository.service.IRepositoryItemService";
var SERVLET_CONFIG_SERVICE_NAME = "com.ibm.team.repository.servlet.internal.ServletConfigurationService";
var ALLOW_ACCCESS_PROPERTY_NAME = "ws.allow.admin.access";
var ADMIN_NOT_DISABLED = "ADMIN access was not disabled.  You need to be authenticated as a user other than ADMIN to disable ADMIN access.";
var ADMIN_ROLES_NOT_DISABLED = "The user roles for ADMIN were not removed.  You need to be authenticated as a user other than ADMIN to remove ADMIN roles.  The ADMIN user may have limited access to the Web UI.";
var CLICK_NEXT = "  Click 'Next' to continue.";
var SELECT_CALS = "Select the appropriate Client Access License(s) to be assigned to your user.";

var REGISTRY_TYPE = {
	DETECT: {configValue: "DETECT", display: "JazzRealm"},
	TOMCAT: {configValue: "TOMCAT", display: "JazzRealm"},
	LDAP: {configValue: "LDAP", display: "LdapRealm"},
	UNSUPPORTED: {configValue: "UNSUPPORTED", display: "UnknownRealm"}
}

var ASSIGN_CLIENT_LICENSES = "AssignClientLicenses";
var LDAP_LICENSE_CHECK = "allows." + REGISTRY_PROPERTY_NAME + "." + REGISTRY_TYPE.LDAP.configValue;
var NO_CLIENT_LICENSES = "There are no Client Access Licenses installed on the server.";
var REVIEW_LIC_ADMIN = "  Review the <a href=\'../jazz/admin#action=com.ibm.team.repository.admin.manageLicenses\'>License Key Management</a> page of the Admin UI for more information. ";
var SEE_DB_CONFIG_MSG = "Complete the <a href=\'#action=com.ibm.team.repository.wizard.configureDatabase\'>Configure Database</a> page to adjust the database settings. "
var SEE_SERVER_STATUS_MSG = "View the <a href=\'../jazz/admin#action=com.ibm.team.repository.admin.serverStatus\'>Admin Server Status Summary</a> for more information.";
var ERROR_FETCHING_LICENSE = "A problem occurred while fetching license information from the repository.";
var NO_LICENSES_ENABLED = "License assignment is not enabled at this time because there are no licenses available.";
var DISABLE_ADMIN_DESCRIPTION = "The default ADMIN user is defined in Jazz to allow you to log into a new server, configure the server, and create an initial JazzAdmins user.  For enhanced server security we recommend that you disable default ADMIN access at this time.";
var CHANGE_ADMIN_PWD = "If you do not disable ADMIN access, you should at least change the default ADMIN password after completing the Jazz Team Server Setup.";
var COMPUTED_LDAP_PROPS = [
	"findGroupsForUserQuery",
	"findUsersByAnyNameQuery",
	"findUsersByNameQuery",
	"findUsersByUserIdQuery"
]

// The SetupUsers widget provides the basic wizard page implementation. It
// provides three User Registry options to the user:
//
// - Jazz Team Server (really tomcat-users.xml accessed via a Jazz UI)
// - LDAP
// - Non-LDAP External Registry 
//
// Once the user selects one of the user registry options, SetupUsers
// instantiates a form to configure the option the user selected in step 1.
// Each form is defined as a separate "class" at the bottom of this module, and
// all subclass SetupUsersDetail (also defined below) to achieve consistent
// behavior.

dojo.declare(PACKAGE_PREFIX + "SetupUsers",  [ConfigUtility, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/SetupUsers.html"),
	 
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.wizard.SetupUsers.superclass.postCreate.apply(this);
		this._navigation = this.wizardNav;
		this._navigation.enablePrevious(true);
		
		// When the next button is activated, this checks to see if LDAP queries shoud be 
		// computed.  Then it saves all configuration changes, updates the status, enables the 
		// next button again, and goes to the next page.
		dojo.connect(this._navigation, "requestNextApproval", this, "_initiateComputeAndSave");
		dojo.connect(this, "_submitConfigCallback", this, "_handleSuccessfulSave"); 
		dojo.connect(this, "_submitConfigError", this, function(){
			this.setStatus({type: "ERROR", message: "An error occurred while saving the configuration properties.  Please check your server log for more details."});
		}); 
		
		// initialize properties
		this.formCache = {};
		this.selectedType = null;
		this._newUserCreated = false;
		this.startLoading();
	}, 
	
	startLoading: function() {	
		// show LDAP as option if allowed by license
		dojo.toggleClass(this.registryTableLdapRow, "noDisplay", !licenseAllowsLdap());			
	
		// set initial status to loading
		this.setStatus({type: "PROGRESS", message: "Loading User Registry settings..."});
		disableRegistryButtons(this.registryTypeTable, true);
		
		// do a remote method to get the status of a service to make the LDAP extension service get loaded
		// the registry type radio buttons are disabled until this call returns to ensure the server is ready
		var handler = new SRH(this, "handleServiceStatusResponse", "postCurrentStateLoadError");
		ServerStatusClient.getServiceStatus(handler, REGISTRY_SERVICE_INTERFACE_NAME);	
	},
	
	handleServiceStatusResponse: function (serviceStatus) {
		// verify that the repo item service is also available before loading the page
		// otherwise the DB is down or has version mismatch
		var handler = new SRH(this, "fetchInitialState", "postCurrentStateLoadError");
		ServerStatusClient.getServiceStatus(handler, REPO_ITEM_SERVICE_INTERFACE_NAME);	
	}, 
	
	fetchInitialState: function (data) {
		if (data.code && data.code === "OK") {
			// shouldn't get here unless the DB status is OK - but contributor may not be in session
			this._noDBAvailable = false;
			var authInSession = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR");
			if (authInSession) {
				this._defaultAdmin = isDefaultAdmin();
				this._loadConfiguration();
			} else {
				var handler = new SRH(this, "handleSimpleLoginResponse", "handleSimpleLoginError");
				AdminClient.getSimpleLogin({}, handler, false);
			}
		} else {
			this._noDBAvailable = true;
			this.setStatus({type: "ERROR", message: "A problem occurred while loading User Registry settings.", disableButtons: true});
			// fetch DB status info to let the user know why the services are not available
			ServerStatusClient.getDatabaseStatus(new SRH(this, "displayDatabaseStatusInfo", "postCurrentStateLoadError"));
		}	
		dojo.toggleClass(this.registryTypeStep,"noDisplay", this._noDBAvailable);
		dojo.toggleClass(this.licenseLimitedMessage, "noDisplay", this._noDBAvailable || licenseAllowsLdap());
		dojo.toggleClass(this.noDatabaseAvailableStep, "noDisplay", !this._noDBAvailable);
		this.createLdapImportInfoBlock();
	},
	
	refreshAvailabilityState: function () {
		if (this._noDBAvailable) {
			// check DB and reload
			this.startLoading();
		}
	},
	
	createLdapImportInfoBlock: function () {
		var args = {
			step: "",
			heading: "Importing Your New User from LDAP",
			description: "Completing the LDAP configuration step above will enable the Jazz Team Server Setup to automatically import your initial user.  Your application server and LDAP server should be configured before you complete this step, and you should be logged into this Setup application as a member of the JazzAdmins role from your LDAP.  When you complete this page and click on the 'Next' button, your user will be imported into the repository.  Any Client Access Licenses you select below will be assigned to your new user."
		};
		var constructor = dojo.getObject(PACKAGE_PREFIX + "StepBox");
		var ldapInfo = new constructor(args);		
		this.ldapImportInfoBlock.appendChild(ldapInfo.domNode);
	},
	
	handleSimpleLoginResponse: function (data) {
		this._defaultAdmin = data && data === "ADMIN";
		this._loadConfiguration();
	},
	
	handleSimpleLoginError: function (err) {
		this.setStatus({type: "ERROR", message: "A problem occurred accessing the user data.", disableButtons: true});		
	} ,
	
	_loadConfiguration: function () {
		// get the current setting for user registry type
		var handler = new SRH(this, "postCurrentStateLoad", "postCurrentStateLoadError");
		ServerStatusClient.getServiceConfiguration(handler, REGISTRY_SERVICE_NAME);
		
		// get the current setting for allowing admin access
		var handler = new SRH(this, "postServletConfigurationLoad", "postCurrentStateLoadError");
		ServerStatusClient.getServiceConfiguration(handler, SERVLET_CONFIG_SERVICE_NAME);	
		
		// fetch the CALs
		if (this._defaultAdmin && !this._newUserCreated) {
			this.getLicensingInfo();
		}
	},
	
	displayDatabaseStatusInfo: function(dbStatus) {
		dojox.data.dom.removeChildren(this.noDatabaseAvailableStep);
		if (dbStatus.code && dbStatus.code === "ERROR") {
			statmessage = "The database service is not available.  Database status: <br>" + dbStatus.message;			
		} else {
			statmessage = "An unknown error occurred.  " + SEE_SERVER_STATUS_MSG;
		}
		this.setStatus({type: "ERROR", message: statmessage, disableButtons: true});
		var args = {
			step: "",
			heading: "Configure Database Connection Before Proceeding",
			description: "The Jazz Team Server is not able to connect to the database.  The server must establish a connection to the database before you can configure the user registry settings.  " + SEE_DB_CONFIG_MSG
		};
		var constructor = dojo.getObject(PACKAGE_PREFIX + "StepBox");
		var dbInfo = new constructor(args);
		this.noDatabaseAvailableStep.appendChild(dbInfo.domNode);	
	},
	
	postCurrentStateLoad: function(currentStateInfo) {
		// Display the current setting for user registry type.  Options:  LDAP, TOMCAT, DETECT, UNSUPPORTED		
		var index = arrayIndexWithPropertyOfObject(currentStateInfo.properties, "name", REGISTRY_PROPERTY_NAME);
		var registryType = currentStateInfo.properties[index];
		this.savedRegistryType = registryType.overrideValue;
		var registryTypeValue = registryType.overrideValue != null ? registryType.overrideValue : registryType.defaultValue;
				
		if (registryTypeValue === REGISTRY_TYPE.DETECT.configValue) { 
			userRegistry.getUserRegistryType(new SRH(this, "postDetectRegistryType", "postCurrentStateLoadError"));
		} else {
			this._setupRegistryType(registryTypeValue);
		}	
		disableRegistryButtons(this.registryTypeTable, false);	
	},
	
	postServletConfigurationLoad: function(servletConfigInfo) {
		// if ADMIN access is not disabled, show checkbox to disable it
		var index = arrayIndexWithPropertyOfObject(servletConfigInfo.properties, "name", ALLOW_ACCCESS_PROPERTY_NAME);
		var allowAdmin = servletConfigInfo.properties[index];
		var allowAdminValue = allowAdmin.overrideValue != null ? allowAdmin.overrideValue : allowAdmin.defaultValue;
				
		if (allowAdminValue === "true") { 
			dojox.data.dom.removeChildren(this._disableAdminArea);
			var content = document.createElement("div"); 
			content.appendChild(document.createTextNode(DISABLE_ADMIN_DESCRIPTION));
			var allowAdminForm = document.createElement("div"); 
			allowAdminForm.className = "spaced-bottom-check";
			this.allowAdminCheck = createInputElement({type: "checkbox", name: "disableAdmin"}); 
			allowAdminForm.appendChild(this.allowAdminCheck);
			this.allowAdminCheck.checked = true;
			this.connect(this.allowAdminCheck,"onclick","handleAdminCheckChange");
			var allowAdminLabel = document.createElement("label");
			allowAdminLabel.htmlFor = "disableAdmin"
			allowAdminLabel.innerHTML = "Disable default ADMIN access";
			allowAdminLabel.className ="spaced";
			allowAdminForm.appendChild(allowAdminLabel);
			content.appendChild(allowAdminForm);
			var args = {
				step: "Step 3",
				heading: "Disable Default ADMIN Access",
				description: content 
			};
			var constructor = dojo.getObject(PACKAGE_PREFIX + "StepBox");
			this.disableAdminForm = new constructor(args);
			this._disableAdminArea.appendChild(this.disableAdminForm.domNode);	
			
			this.adminPasswordInfoNode = document.createElement("div");
			this.adminPasswordInfoNode.className = "spaced-all warning-text noDisplay";
			this.adminPasswordInfoNode.innerHTML = CHANGE_ADMIN_PWD;
			this.allowAdminCheck.parentNode.appendChild(this.adminPasswordInfoNode);
			
		} else {
			this._skipStep3 = true;
		} 
	},
	
	handleAdminCheckChange: function () {
		dojo.toggleClass(this.adminPasswordInfoNode, "noDisplay", this.allowAdminCheck.checked);
	},
	
	_setupRegistryType: function(type) {
		setSelectedRegistryButton(this.registryTypeTable, type);
		this.displaySelectedRegistryType(type);
	}, 
	
	postDetectRegistryType: function (type) {
		//TODO ignore case in case of Tomcat
		if (type === "tomcat") {
			targetType = REGISTRY_TYPE.TOMCAT.configValue;
		} else if (licenseAllowsLdap()){
			targetType = REGISTRY_TYPE.LDAP.configValue;
		} else {
			targetType = REGISTRY_TYPE.UNSUPPORTED.configValue;
		}
		this._setupRegistryType(targetType);
	},
	
	postCurrentStateLoadError: function(errorObj) {
		this.setStatus({type: "ERROR", message: "A problem occurred while loading User Registry settings.", disableButtons: true});
	},
	
	onSelectRegistryType: function(e) {
		if(!e.target || e.target.tagName !== "INPUT") {return;}
		
		var selected = getSelectedRegistryType(this.registryTypeTable);
		this.displaySelectedRegistryType(selected);
	},
	
	displaySelectedRegistryType: function(type) {
		this.formContainer.className = "net-jazz-ajax-block";
		this.selectedType = type;
		var formType = REGISTRY_TYPE[type].display;
		if (!this._defaultAdmin || this._newUserCreated) {
			this.hideClientLicenseForm();
			if (type != REGISTRY_TYPE.LDAP.configValue) {
				formType = "NoNewUserRequired";
			}
		}
		var showLdapImport = type === REGISTRY_TYPE.LDAP.configValue && this._defaultAdmin && !this._newUserCreated;
		dojo.toggleClass(this.ldapImportInfoBlock,"noDisplay", !showLdapImport);

		var widget = this.formCache[formType];
		if(!widget) {
			var constructor = dojo.getObject(PACKAGE_PREFIX + formType);
			var args = {
				step: "Step 2",
				setStatusCallback: dojo.hitch(this, this.setStatus)
			};
			widget = new constructor(args);
			this.formCache[formType] = widget;
		}
		this.currentForm = widget;
		if (widget.validate != null) {
			widget.validate();
		}
		
		if(this.formContainer.firstChild){
			if(this.formContainer.firstChild !== widget.domNode) {
				this.formContainer.replaceChild(widget.domNode, this.formContainer.firstChild);
			}
		} else{
			this.formContainer.appendChild(widget.domNode)
		}
	},
	
	hideClientLicenseForm: function () {
		dojox.data.dom.removeChildren(this._licenseAssignmentList);
		this.licenseForm = null;
	}, 
	
	setStatus: function(status) {
		var canClickNext = false;
		if (status.type === "OK" || (status.overrideNext != null && status.overrideNext)) {
			canClickNext = true;
		}
		this._navigation.enableNext(canClickNext);
		if (!status.disableButtons) {
			disableRegistryButtons(this.registryTypeTable, false);
		}
		if(canClickNext && status.type === "OK") {
			status.message = "You have successfully configured the User Registry." + CLICK_NEXT;
		}
		this.statusBox.className = "statusBox " + status.type + "2";
		this.statusBox.innerHTML = status.message;
	}, 
	
	// Licensing 
	getLicensingInfo: function () {
		LicenseAdminClient.getLicenseInfo(new SRH(this, "getLicensingInfoCallback", "handleLicensingInfoError"));
	},
			
	getLicensingInfoCallback: function (data) {
		dojox.data.dom.removeChildren(this._licenseAssignmentList);
		var args = {
			step: this._skipStep3 ? "Step 3" : "Step 4",
			heading: "Assign Client Access Licenses",
			description: SELECT_CALS
		};
		var enabled = false;
		if (data.types != null) {
			var licenses = LicenseAdminClient.sortCALs(data.types);
			licenses = filterOutFunctionalCALs(licenses);
			var licenseRowData = [];
			for (var i=0; i < licenses.length; i++) {
				var CALInfo = LicenseAdminClient.getClientAccessLicenseInfo(data, licenses[i])
				var licTypeNode = document.createElement("div");
				licTypeNode.innerHTML = "<label for='" + licenses[i].id + "'>" + CALInfo.name + "</label>";
				var descriptionNode = document.createElement("div");
				descriptionNode.className = "inline-description";
				descriptionNode.innerHTML = licenses[i].description;
				licTypeNode.appendChild(descriptionNode);
				
				var disableCAL = !CALInfo.assignableToUserWithoutCALs;
				licenseRowData.push({
					name: licenses[i].id, 
					type: "checkbox", 
					label: licTypeNode, 
					description: licenses[i].description,
					availability: CALInfo.simpleAvailabilityString,
					disabled: disableCAL
				});		
				if (!disableCAL){
					enabled = true;
				}
			}
			if (!LicenseAdminClient.isServerAcceptingNewLicensedUsers(data)) {	
				var descNode = document.createElement("div");
				var descTextNode = document.createElement("div");
				descTextNode.innerHTML = enabled? args.description: NO_LICENSES_ENABLED;
				descNode.appendChild(descTextNode);
				var maxNode = this.createWarningNode(LicenseAdminClient.SERVER_COUNT_MAX_MSG + REVIEW_LIC_ADMIN);
				descNode.appendChild(maxNode);
				args.description = descNode;
			} else if (!enabled){
				args.description = this.createWarningNode(NO_LICENSES_ENABLED + REVIEW_LIC_ADMIN);
			}
			args.tableContent = { header: CLIENT_LICENSE_HEADER_DEF, rows: licenseRowData};	
		}
		else {
			args.description = this.createWarningNode(NO_CLIENT_LICENSES + REVIEW_LIC_ADMIN);
		}
		var constructor = dojo.getObject(PACKAGE_PREFIX + "StepBox");
		this.licenseForm = new constructor(args);
		this._licenseAssignmentList.appendChild(this.licenseForm.domNode);	
	},

	createWarningNode: function(message) {
		var warningNode = document.createElement("div");
		warningNode.className = "WARNING spaced-all";
		warningNode.innerHTML = message;
		return warningNode;
	}, 
	
	handleLicensingInfoError: function(errorObj) {
		this.setStatus({type: "ERROR", message: ERROR_FETCHING_LICENSE});
		var constructor = dojo.getObject(PACKAGE_PREFIX + "StepBox");
		var args = {
			step: this._skipStep3 ? "Step 3" : "Step 4",
			heading: "Assign Client Access Licenses",
			description: ERROR_FETCHING_LICENSE + "  See the server log for details."
		};
		this.licenseForm = new constructor(args);
		this._licenseAssignmentList.appendChild(this.licenseForm.domNode);	
	},
	
	// handle Users
	_fetchItemIdForUserId: function(user) {
		var args = {
			userId: user
		}
		RepositoryClient.getContributors(new SRH(this, "postContributorSearchResult", "postContributorSearchError"), args);
	}, 
	
	postContributorSearchResult: function(contributor) {
		if (contributor && contributor[0].itemId) {
			var licenseChanges = this._getLicenseChanges();
			this.currentForm.runAction(contributor[0].itemId, licenseChanges, this);
		} else {
			this.setStatus({type: "WARNING", message: "You need to be authenticated as a user from LDAP to import your user and assign licenses."});
		}	
	}, 
	
	postContributorSearchError: function(errorObj) {
		this.setStatus({type: "ERROR", message: "A problem occurred looking up the new user."});
	},
	
	// SUPER ConfigUtility METHODS	
	_getChanges: function () {
		var changes = [];
		var properties = [];
		var registryType = this.selectedType != null ? this.selectedType : REGISTRY_TYPE.DETECT.configValue;
		if (registryType === REGISTRY_TYPE.LDAP.configValue && this.currentForm.ldapConfig._changed) {
			var changeObj = this.currentForm.ldapConfig._getChanges();
			changes = changeObj["changes"];
			properties = changeObj["properties"];
		}
		if (this.savedRegistryType != registryType) {
			var registryChange = {
				name: REGISTRY_PROPERTY_NAME,
				value: registryType,
				serviceImplementationName: REGISTRY_SERVICE_NAME
			};
			changes.push(registryChange);
		}
		return {
			"changes": changes,
			"properties": properties
		};

	},
	
	// SAVE METHODS
	_initiateComputeAndSave: function() {
		this._navigation.enableNext(false);
		this.setStatus({type: "PROGRESS", message: "Saving settings..."});
		if (this.selectedType == null) {
			this._handleInvalidRegistrySelection();
			return;
		}
		disableRegistryButtons(this.registryTypeTable, true);
		if (this.selectedType === REGISTRY_TYPE.LDAP.configValue) {
			this._computeLdapQueries();
		} else {
			this._validateNonLdapRegistry();
		}	
	}, 

	_validateNonLdapRegistry: function() {
		var status = this.currentForm.validate();
		if(status.isValid && status.isComplete){
			this._invokeSaveAfterCompute();
		} else {
			this.setStatus(status);
		}
	},
	
	_computeLdapQueries: function() {
		if (this.currentForm.computeLdapOption.checked) {			
			var pendingUpdates = this.currentForm.ldapConfig._getAllValuesExceptSensitiveReplacements();
			if (pendingUpdates.length === 0) {
				return false;
			}
			var properties = {};
			for (var i=0; i<pendingUpdates.length; i++) {
				var name = pendingUpdates[i].name;
				var value = pendingUpdates[i].value;
				properties[name] = value;
			};
			
			var handler = new SRH(this, "cacheComputedQueries", "postComputeQueryError");
			userRegistry.computeLDAPQueries(handler, {basicProperties: dojo.toJson(properties)});
		} else {
			this._invokeSaveAfterCompute();
		}
	},
	
	cacheComputedQueries: function (results) {	
		var ldapProperties = this.currentForm.ldapConfig;	
		for (var i=0; i<COMPUTED_LDAP_PROPS.length; i++) {
			// find the prop in ldap props with matching name.  set it's value and changed flag
			var index = arrayIndexWithPropertyOfObject(ldapProperties._properties, "_name", "ldap." + COMPUTED_LDAP_PROPS[i]);
			var propertyElement = ldapProperties._properties[index];
			if (propertyElement._editValueNode.value != results[COMPUTED_LDAP_PROPS[i]]) {
				propertyElement._editValueNode.value = results[COMPUTED_LDAP_PROPS[i]];
				propertyElement._changed = true;
				ldapProperties._changed = true;
				propertyElement._validationPending = false;
			}
		};
		
		this._invokeSaveAfterCompute();
	}, 
	
	postComputeQueryError: function(errorObj) {
		this.setStatus({type: "ERROR", message: "A problem occurred while computing LDAP queries."});
	},
	
	_invokeSaveAfterCompute: function () {
		// If any configuration properties need to be saved, _handleSuccessfulSave will be called
		// when the updates are complete.  Otherwise, proceed to create the new user
		var invokedSave = this._saveHandler();
		if (!invokedSave) {
			this._initiateCreateUser();
		}
	}, 
	
	_handleSuccessfulSave: function() {
		this.savedRegistryType = this.selectedType != null ? this.selectedType : REGISTRY_TYPE.DETECT.configValue;
		this._initiateCreateUser();	
	},
	
	_initiateCreateUser: function () {
		if (!this._defaultAdmin || this._newUserCreated){
			this.disableAdminAccess();
			return;
		}
		if (this.savedRegistryType === REGISTRY_TYPE.LDAP.configValue) {
				userRegistry.importUserFromRegistry(new SRH(this, "postCreateCallback", "postImportUserError"));
		} else {
			var valueMap = this.currentForm.getData();
			var licenseChanges = this._getLicenseChanges();
			this.currentForm.runAction(valueMap, licenseChanges, this);			
		}
	},
	
	_getLicenseChanges: function() {
		var licenseChanges = {
			add: [],
			remove: []
		};
		if (this.licenseForm && this.licenseForm.getData()) {
			var assigned = this.licenseForm.getData();
			for (i in assigned) {
				licenseChanges[(assigned[i] ? "add" : "remove")].push(i);
			}
		}
		return licenseChanges;
	},
	
	postCreateCallback: function(data) {
		this._newUserCreated = true;
		if (this.savedRegistryType === REGISTRY_TYPE.TOMCAT.configValue) {
			var newUserId = this.currentForm.getData().userId;
			var newPassword = this.currentForm.getData().password;
			AdminClient.switchLogin(newUserId, newPassword, new SRH(this, "disableAdminAccess", "postLoginErrorCallback"));
		} else if (this.savedRegistryType === REGISTRY_TYPE.LDAP.configValue) { 
			if (data){
				this._fetchItemIdForUserId(data);
			} else {
				this.setStatus({type: "WARNING", message: "You need to be authenticated as a user from LDAP to import your user and assign licenses."});
			}
		} else {
			this.disableAdminAccess();
		}		
	},
	
	disableAdminAccess: function () {
		// call this after switching to new user 
		if (this.allowAdminCheck != null) {
			// get user, verify not ADMIN
			var handler = new SRH(this, "checkLoginDisableAdminResponse", "handleSimpleLoginError");
			AdminClient.getSimpleLogin({}, handler, false);
		} else {
			this.postFinishSetupUsers();
		}
	},
	
	checkLoginDisableAdminResponse: function (data) {
		//verify user not ADMIN
		this._defaultAdmin = data && data === "ADMIN";		
		if (!this._defaultAdmin){
			var newValue = !this.allowAdminCheck.checked ? "true" : "false";
			var propUdpate = [{
				name: ALLOW_ACCCESS_PROPERTY_NAME,
	 			value: newValue,
	 			serviceImplementationName: SERVLET_CONFIG_SERVICE_NAME     
			}];
			var changeArgs = {
				updates: dojo.toJson(propUdpate)
			};
			var srh = new SRH(this, "postRemoveAdminRoles", "postAdminAccessError");
			ServerStatusClient.postConfigurationUpdates(srh, changeArgs);
		} else {
			if (this.allowAdminCheck.checked) {
				// should be new user logged in, but still ADMIN
				this.setStatus({type: "WARNING", message: ADMIN_NOT_DISABLED + CLICK_NEXT, overrideNext: true});
				this.allowAdminCheck.checked = false;
				this.handleAdminCheckChange();
				this.allowAdminCheck.disabled = true;
				
				var descNode = document.createElement("div");
				descNode.className = "WARNING spaced-all";
				descNode.innerHTML = ADMIN_NOT_DISABLED;
				this.allowAdminCheck.parentNode.insertBefore(descNode, this.allowAdminCheck);
			} else {
				this.postFinishSetupUsers();
			}
		} 
	},
	
	postRemoveAdminRoles: function () {
		if (!this._defaultAdmin) {
			if (this.allowAdminCheck.checked){
				// if allow admin access was disabled, also remove the roles for ADMIN
				AdminClient.postRemoveAdminRoles(new SRH(this, "postFinishSetupUsers", "postAdminRolesError"));
			} else {
				// admin access wasn't disabled, so continue on
				this.postFinishSetupUsers();
			}
		} else {
			// shouldn't be able to get here as ADMIN user
			this.setStatus({type: "WARNING", message: ADMIN_ROLES_NOT_DISABLED + CLICK_NEXT, overrideNext: true});
		}
	},
	
	postLoginErrorCallback: function (err) {
		if (err.message != null) {
			this.setStatus({type: "WARNING", message: "There was an error attempting to login as the new user: " + err.message, overrideNext: true});
		} else {
			this.setStatus({type: "WARNING", message: "There was an error attempting to login as the new user.", overrideNext: true});
		}
	},
	
	postCreateErrorCallback: function (err) {
		if (err.message != null) {
			this.setStatus({type: "ERROR", message: "There was an error attempting to create the new user: " + err.message});
		} else {
			this.setStatus({type: "WARNING", message: "There was an unknown error attempting to create the new user.  Please check your server configuration.", overrideNext: true});
		}
	},
	
	postFinishSetupUsers: function() {
		this.displaySelectedRegistryType(this.savedRegistryType);
		this._navigation.processNext();
		this.setStatus({type: "OK"});
	},
	
	postAssignLicensesError:  function(error) {
		this.setStatus({type: "WARNING", 
			message: "There was an error assigning licenses the new user: " + error.errorMessage, 
			overrideNext: true});
	},
	
	postAdminAccessError:  function(error) {
		this.setStatus({type: "WARNING", 
			message: "There was an error disabling ADMIN access: " + error.errorMessage, 
			overrideNext: true});
	},	
	
	postAdminRolesError:  function(error) {
		this.setStatus({type: "WARNING", 
			message: "There was an error removing the roles for ADMIN: " + error.errorMessage, 
			overrideNext: true});
	},	
	
	postImportUserError: function(error) {
		this.setStatus({type: "WARNING", 
			message: "There was an error attempting to create the new user: " + error.errorMessage, 
			overrideNext: true});
	},
	
	_handleInvalidRegistrySelection: function() {
		this.setStatus({type: "INFO", message: "Please select a User Registry type from the list."});
	}
	
});

var VALIDATORS = {
	email: {
		regex: new RegExp(dojox.regexp.emailAddress()),
		message: "Invalid e-mail address"
	}
};

var CREATE_USER_HEADER_DEF = [
	{type: "label", label: "Property"},
	{type: "data", label: "Value"},
	{type: "description", label: "Description"}
];

var USER_ID_FIELD = {name: "userId", label: "User ID", description: "Your user ID (e.g. 'jsmith')"};
var NAME_FIELD = {name: "name", label: "Name", description: "Your full name (e.g. 'John Smith')"};
var EMAIL_ADDRESS_FIELD = {name: "emailAddress", label: "E-mail Address", dataType: "email", description: "Your e-mail address (e.g. 'jsmith@example.com')"};

var JAZZ_REALM_TABLE = {
	header: CREATE_USER_HEADER_DEF,
	rows: [
		USER_ID_FIELD,
		NAME_FIELD,
		{name: "password", label: "Password", type: "password", description: "Enter a password that you can remember but won't be easy for others to guess."},
		{name: "repeatPassword", label: "Re-type Password", type: "password", description: "Re-type your password to help prevent typos.", mustMatch: {
			matchName: "password",
			errorMessage: "Passwords don't match"
		}},
		EMAIL_ADDRESS_FIELD
	]
};

var UNKNOWN_REALM_TABLE = {
	header: CREATE_USER_HEADER_DEF,
	rows: [
		USER_ID_FIELD,
		NAME_FIELD,
		EMAIL_ADDRESS_FIELD
	]
};

var CLIENT_LICENSE_HEADER_DEF = [
	{type: "data", label: "", className: "slight-tight-cell"},
	{type: "label", label: "Type"},
	{type: "availability", label: "Availability",className: "wizardTypeCell"}
];

function getSelectedRegistryType(registryTypeTable) {
	var inputs = registryTypeTable.getElementsByTagName("input");
	var value = null;
	for (var i=0; i<inputs.length; i++) {
		if (inputs[i].checked) {
			value = inputs[i].value;
			break;
		}
	}
	return value;
}

function setSelectedRegistryButton(registryTypeTable, target) {
	var inputs = registryTypeTable.getElementsByTagName("input");
	for (var i=0; i<inputs.length; i++) {
		if (inputs[i].value === target) {
			inputs[i].checked = true;
			break;
		}
	}
}

function disableRegistryButtons(registryTypeTable, disable) {
	var inputs = registryTypeTable.getElementsByTagName("input");
	for (var i=0; i<inputs.length; i++) {
		inputs[i].disabled = disable;
	}
}

function licenseAllowsLdap() {
	return (dojo.getObject(LDAP_LICENSE_CHECK) != null && dojo.getObject(LDAP_LICENSE_CHECK));
}

function filterOutFunctionalCALs(calList){
	var filteredCALs = [];
	dojo.forEach(calList, function(cal){
		if (!cal.functional) {
			filteredCALs.push(cal);
		}
	});
	return filteredCALs;
}	

// The SetupUsersDetail is the common superclass for different forms used to
// configure the different types of user registries (Jazz Team Server, LDAP,
// non-LDAP external registry). The key value add of this class is to provide a
// common data validation routine that introspects the form's data and metadata
// to determine if it's ready to be submitted.

dojo.declare(PACKAGE_PREFIX + "SetupUsersDetail",  [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/SetupUsersDetail.html"),

	postCreate: function(){
		if(isNode(this.description)) {
			this.descriptionArea.appendChild(this.description);
		} else {
			this.descriptionArea.innerHTML = this.description;
		}
		this.descriptionArea.className = "description-area";
		
		if (!this.tableContent) {
			dojo.toggleClass(this.OuterPropertyTable, "noDisplay", true);
			return;
		}
		this.connect(this.propertyTableBody,"onkeyup","handleKeyUp");
		
		var headerObjs = this.tableContent.header;
		for(var i=0; i<headerObjs.length; i++) {
			var hObj = headerObjs[i];
			var th = document.createElement("th");
			if(hObj.type === "data"){
				th.className = "data";
			} 
			if(hObj.className){
				th.className = hObj.className;
			}
			this.propertyTableHeaderRow.appendChild(th);
			th.appendChild(document.createTextNode(hObj.label));
		}
		
		var tableDefRows = this.tableContent.rows;
		this.dataFields = [];
		var inputMap = {};
		var mustMatchMap = {};
		for(var i=0; i<tableDefRows.length; i++){
			var row = document.createElement("tr");
			var tableDefRow = tableDefRows[i];
			this.propertyTableBody.appendChild(row);
			for (var j=0; j<headerObjs.length; j++) {
				var colType = headerObjs[j].type;
				var td = row.insertCell(-1);
				if(colType === "data"){
					td.className = "data";
					var input = createInputElement(tableDefRow);
					this.dataFields.push(input);
					td.appendChild(input);
					this.connect(input,"onblur","handleChange");
					// store a back-pointer to containing row
					input._jazzRow = row;
					// if this field must match another, store a reference in
					// the 'must match' map for later
					var name = tableDefRow.name;
					inputMap[name] = input;
					if(tableDefRow.mustMatch){
						mustMatchMap[name] = tableDefRow.mustMatch;
					}
					// if this field has a particular data type, store a
					// reference to the relevant validator
					if(tableDefRow.dataType){
						input._jazzValidator = VALIDATORS[tableDefRow.dataType];
					}
				} else {
					var val = tableDefRow[colType];
					if (colType === "description" ) {
						td.className = "inline-description";
					} 
					if(isNode(val)) {
						var node = val;
					} else {
						node = document.createTextNode(val);
					}
					td.appendChild(node);
				}
			}
		}
		// for each member of the mustMatchMap, create a reference in the source input
		// to the input the source input must match
		for (var inputName in mustMatchMap) {
			var srcInput = inputMap[inputName];
			var matchEntry = mustMatchMap[inputName];
			matchEntry.matchField = inputMap[matchEntry.matchName];
			
			srcInput._jazzMustMatch = matchEntry;
		}
	},
	
	handleKeyUp: function(e) {
		if(e.target && e.target.tagName === "INPUT") {
			if(this.onKeyUpTimout){
				clearTimeout(this.onKeyUpTimout);
			}
			var f = dojo.hitch(this, this.validate);
			var timeout = 2000;
			if (e.target._jazzRow  && e.target._jazzRow.hasError) {
				timeout = 500;
			}
			this.onKeyUpTimout = setTimeout(f, timeout);
		}
	},
	
	handleChange: function(e) {
		this.validate();
	},
	
	validate: function() {
		var formStatus = {
			isComplete: true,
			isValid: true,
			type: "INFO"
		};
		if (this.dataFields) {
			for (var i = 0; i < this.dataFields.length; i++) {
				var input = this.dataFields[i];
				var fieldStatus = checkField(input);
				if (fieldStatus.isEmpty) {
					formStatus.isComplete = false;
				}
				var td = input.parentNode;
				var msgDiv = td.childNodes.length > 1 ? td.childNodes[1] : null;
				if (fieldStatus.isValid) {
					dojo.removeClass(input._jazzRow, "hasError");
					input._jazzRow.hasError = false;
					if (msgDiv) {
						td.removeChild(msgDiv);
					}
				}
				else {
					formStatus.isValid = false;
					formStatus.type = "ERROR";
					dojo.addClass(input._jazzRow, "hasError");
					input._jazzRow.hasError = true;
					if (!msgDiv) {
						msgDiv = document.createElement("div");
					}
					else {
						msgDiv.innerHTML = "";
					}
					msgDiv.appendChild(document.createTextNode(fieldStatus.error));
					td.appendChild(msgDiv);
				}
			}
		}
		clearTimeout(this.onKeyUpTimout);
		formStatus.overrideNext = formStatus.isComplete && formStatus.isValid;
		return formStatus;
	},
	
	getData: function() {
		var obj = {};
		if (!this.dataFields) {
			return obj;
		}
		for(var i=0; i<this.dataFields.length; i++){
			var name = this.dataFields[i].name;
			if (this.dataFields[i].type == "checkbox") {
				obj[name] = this.dataFields[i].checked;
			} else {
				obj[name] = dojo.trim(this.dataFields[i].value);
			}
		}
		return obj;
	}
});

function checkField(input) {
	var isValid = true;
	var isEmpty = input.value === "";
	var error = null;
	
	// check if the field must match another field and if so check it
	if(isValid && !isEmpty && input._jazzMustMatch){
		var mustMatchValue = input._jazzMustMatch.matchField.value;
		if(input.value !== dojo.trim(mustMatchValue)){
			isValid = false;
			error = input._jazzMustMatch.errorMessage;
		}
	}
	
	// check to see if the field has any validator and if so if it's valid
	if(isValid && !isEmpty && input._jazzValidator){
		var match = input._jazzValidator.regex.exec(input.value);
		if(!match){
			isValid = false;
			error = input._jazzValidator.message;
		}
	}
	
	return {
		isValid: isValid,
		isEmpty: isEmpty,
		error: error
	};
}

// The following two classes provide implementations for the different user
// registry types (Jazz Team Server, Non-LDAP External Registry). They
// subclass SetupUsersDetail for common validation logic, and SetupUsersDetail
// calls down to these classes' runAction method when the user presses the
// form's action button (e.g. "Create User", "Test connection").

var CREATE_USER_MESSAGE = "Please complete the form to create a new user and press the 'Next' button.";

var SETUP_USERS_DETAIL = com.ibm.team.repository.web.ui.internal.wizard.SetupUsersDetail;

dojo.declare(PACKAGE_PREFIX + REGISTRY_TYPE.TOMCAT.display, SETUP_USERS_DETAIL, {
	
	// properties
	heading: "Create a user account for yourself",
	description: "You are currently logged in as the default user (ADMIN). Please create a user account for yourself using the form below. When you press the \"Next\" button, the new user account will be created in the Jazz Team Server.  Your new account will have the JazzAdmins role, which will allow you to login to the Admin UI.",
	tableContent: JAZZ_REALM_TABLE,
		
	runAction: function(valueMap, licenses, caller) {
		// alert("Run " + REGISTRY_TYPE.TOMCAT.display + " action {" + objToStr(valueMap) + "}");
		var args = new Object();
		for (i in valueMap) {
			if (i == "repeatPassword") {
				continue;
			}
			args[i] = valueMap[i];
		}
		args["itemId"] = "new";
		args["jsonRoles"] = dojo.toJson(["JazzAdmins"]);
		args["jsonLicenses"] = dojo.toJson(licenses);

		AdminClient.postContributor(args, new SRH(caller, "postCreateCallback", "postCreateErrorCallback"));
	},
	
	validate: function() {
		var status = this.inherited("validate", arguments);
		if (!status.isValid) {
			status.message = "Please correct the fields below.";
		} else {
			status.message = CREATE_USER_MESSAGE;
		}
		this.setStatusCallback(status);
		return status;
	}
});

dojo.declare(PACKAGE_PREFIX + REGISTRY_TYPE.UNSUPPORTED.display, SETUP_USERS_DETAIL, {
	
	heading: "Create a user for yourself in Jazz Team Server",
	description: "Please use the form below create a user for yourself in Jazz Team Server. Your Jazz user ID is already set to your login ID.",
	tableContent: UNKNOWN_REALM_TABLE,
		
	runAction: function(valueMap, licenses, caller) {
		//alert("Run " + REGISTRY_TYPE.UNSUPPORTED.display + " action {" + objToStr(valueMap) + "}");
		var args = new Object();
		for (i in valueMap) {
			args[i] = valueMap[i];
		}
		args["itemId"] = "new";
		args["repoOnly"] = true; 
		args["jsonLicenses"] = dojo.toJson(licenses);

		AdminClient.postContributor(args, new SRH(caller, "postCreateCallback", "postCreateErrorCallback"));
	},
	
	validate: function() {
		var status = this.inherited("validate", arguments);
		if (!status.isValid) {
			status.message = "Please correct the fields below.";
		} else {
			status.message = CREATE_USER_MESSAGE;
		}
		this.setStatusCallback(status);
		return status;
	}
});

dojo.declare(PACKAGE_PREFIX + "NoNewUserRequired", SETUP_USERS_DETAIL, {
	
	heading: "Your user exists in Jazz Team Server",
	description: "You have successfully created a new user with the JazzAdmins role.",
	
	validate: function() {
		var status  = {
			type: "OK",
			isComplete: true,
			isValid: true
		};
		this.setStatusCallback(status);
		return status;
	}, 
	
	runAction: function(valueMap, licenses, caller) {
		caller.disableAdminAccess();
	}	
});

dojo.declare(PACKAGE_PREFIX + "StepBox", SETUP_USERS_DETAIL, {
});

function isNode(o){
	return o && o.nodeType && o.cloneNode;
}

function createInputElement(args){
	var input = document.createElement("input");
	// set the type of the field if necesary
	if (args.type) {
		input.type = args.type;
	}
	// disable the field if necessary
	if (args.disabled) {
		input.disabled = true;
	}
	input.name = args.name;
	if (args.type === "checkbox") {
		input.id = args.name;
	}
	return input;
}

})();